@@ -48,6 +48,7 @@ struct _DexThreadPoolWorker
 
   GThread                   *thread;
   GMainContext              *main_context;
+  GMainLoop                 *main_loop;
   DexAioContext             *aio_context;
   DexWorkQueue              *global_work_queue;
   DexWorkStealingQueue      *work_stealing_queue;
@@ -78,6 +79,14 @@ static void     dex_thread_pool_worker_set_remove        (DexThreadPoolWorkerSet
 static GSource *dex_thread_pool_worker_set_create_source (DexThreadPoolWorkerSet *set,
                                                           DexThreadPoolWorker    *thread_pool_worker);
 
+static gboolean
+dex_thread_pool_worker_work_item_cb (gpointer user_data)
+{
+  DexWorkItem *work_item = user_data;
+  dex_work_item_invoke (work_item);
+  return G_SOURCE_REMOVE;
+}
+
 static void
 dex_thread_pool_worker_push (DexScheduler *scheduler,
                              DexWorkItem   work_item)
@@ -86,10 +95,36 @@ dex_thread_pool_worker_push (DexScheduler *scheduler,
 
   g_assert (DEX_IS_THREAD_POOL_WORKER (thread_pool_worker));
 
-  if G_LIKELY (g_thread_self () == thread_pool_worker->thread)
+  if G_LIKELY (g_thread_self () == thread_pool_worker->thread &&
+               thread_pool_worker->status == DEX_THREAD_POOL_WORKER_RUNNING)
     dex_work_stealing_queue_push (thread_pool_worker->work_stealing_queue, work_item);
   else
-    dex_work_queue_push (thread_pool_worker->global_work_queue, work_item);
+    {
+      GSource *source;
+
+      /* Pushing a work item directly onto a worker is generally going
+       * to be related to completing work items. Treat those as extremely
+       * high priority as they will delay further processing of futures.
+       *
+       * This is currently a workaround to improve the situation with
+       * issue #17 when we're on fallback configurations. If we want
+       * to improve that situation further, we can reduce some overhead
+       * here by creating a dedicated GSource like DexMainScheduler does
+       * for work items instead of a 1:1 GSource creation.
+       *
+       * But this at least improves the situation immediately that is
+       * causing potential lock-ups.
+       */
+
+      source = g_idle_source_new ();
+      g_source_set_priority (source, G_MININT);
+      g_source_set_callback (source,
+                             dex_thread_pool_worker_work_item_cb,
+                             g_memdup2 (&work_item, sizeof work_item),
+                             g_free);
+      g_source_attach (source, thread_pool_worker->main_context);
+      g_source_unref (source);
+    }
 }
 
 static gboolean
@@ -100,6 +135,7 @@ dex_thread_pool_worker_finalize_cb (gpointer data)
 
   /* First set the status to ensure that our check/dispatch will bail */
   thread_pool_worker->status = DEX_THREAD_POOL_WORKER_STOPPING;
+  g_main_loop_quit (thread_pool_worker->main_loop);
 
   /* Now flush out the rest of the work items if there are any */
   while (dex_work_stealing_queue_pop (thread_pool_worker->work_stealing_queue, &work_item))
@@ -148,6 +184,7 @@ dex_thread_pool_worker_finalize (DexObject *object)
 
   g_clear_pointer (&thread_pool_worker->thread, g_thread_unref);
   g_clear_pointer (&thread_pool_worker->main_context, g_main_context_unref);
+  g_clear_pointer (&thread_pool_worker->main_loop, g_main_loop_unref);
   g_clear_pointer (&thread_pool_worker->work_stealing_queue, dex_work_stealing_queue_unref);
 
   dex_clear (&thread_pool_worker->global_work_queue);
@@ -238,6 +275,7 @@ dex_thread_pool_worker_thread_func (gpointer data)
     {
       thread_pool_worker->status = DEX_THREAD_POOL_WORKER_FINISHED;
       g_cond_signal (&thread_pool_worker->setup_cond);
+      g_mutex_unlock (&thread_pool_worker->setup_mutex);
       return NULL;
     }
 
@@ -287,8 +325,7 @@ dex_thread_pool_worker_thread_func (gpointer data)
   g_mutex_unlock (&thread_pool_worker->setup_mutex);
 
   /* Process main context until we are told to shutdown */
-  while (thread_pool_worker->status != DEX_THREAD_POOL_WORKER_STOPPING)
-    g_main_context_iteration (thread_pool_worker->main_context, TRUE);
+  g_main_loop_run (thread_pool_worker->main_loop);
 
   /* Discard our work queue loop */
   dex_clear (&global_work_queue_loop);
@@ -481,6 +518,7 @@ dex_thread_pool_worker_new (DexWorkQueue           *work_queue,
 
   thread_pool_worker = (DexThreadPoolWorker *)dex_object_create_instance (DEX_TYPE_THREAD_POOL_WORKER);
   thread_pool_worker->main_context = g_main_context_new ();
+  thread_pool_worker->main_loop = g_main_loop_new (thread_pool_worker->main_context, FALSE);
   thread_pool_worker->global_work_queue = dex_ref (work_queue);
   thread_pool_worker->work_stealing_queue = dex_work_stealing_queue_new (255);
   thread_pool_worker->set = set;
