@@ -408,6 +408,7 @@ G_PASTE (test_complete_, T) (DexFuture *future, gpointer user_data) \
   g_assert_error (error, DEX_ERROR, DEX_ERROR_DEPENDENCY_FAILED); \
   g_assert_null (value); \
   g_main_loop_quit (main_loop); \
+  g_error_free (error); \
   return NULL; \
 } \
 static void \
@@ -871,7 +872,7 @@ static void
 test_delayed_simple (void)
 {
   DexFuture *result = dex_future_new_for_int (123);
-  DexFuture *delayed = dex_delayed_new (dex_ref (result));
+  DexFuture *delayed = dex_delayed_new (result);
 
   ASSERT_STATUS (result, DEX_FUTURE_STATUS_RESOLVED);
   ASSERT_STATUS (delayed, DEX_FUTURE_STATUS_PENDING);
